/*
 * Toshiba T1000 Keyboard interpreter
 * 
 * Converts signals from a T1000 keyboard's key matrix to a stndard USB keyboard when using a Teensy LC.
 * 
 * 2017 Chad Boughton
 */

#include <Keypad.h>

const byte ROWS = 8; //eight rows
const byte COLS = 16; //sixteen columns
char keys[ROWS][COLS] = {
  {'1','2',KEY_ESC,'1',KEY_F4,KEY_G,KEY_PAGE_DOWN,KEY_H,KEY_F7,KEY_F10,KEY_QUOTE,KEY_PAGE_UP,'1',KEY_DOWN,'3','a'},
  {'1',KEY_LEFT_SHIFT,KEY_TAB,KEY_F1,KEY_F3,KEY_T,KEY_RIGHT_BRACE,KEY_Y,KEY_F6,KEY_F9,KEY_LEFT_BRACE,KEY_UP,'1','2','3','a'},
  {'1','2','3','1',KEY_F2,KEY_5,KEY_EQUAL,KEY_6,KEY_F5,KEY_F8,KEY_MINUS,'3','1',KEY_PRINTSCREEN,'3','a'},
  {'1','2',KEY_1,KEY_2,KEY_3,KEY_4,KEY_LEFT,KEY_7,KEY_8,KEY_9,KEY_0,KEY_SCROLL_LOCK,'1','2','3','a'},
  {'1','2',KEY_Q,KEY_W,KEY_E,KEY_R,KEY_EQUAL,KEY_U,KEY_I,KEY_O,KEY_P,'3',KEY_END,KEY_HOME,'3','a'},
  {'1','2',KEY_A,KEY_S,KEY_D,KEY_F,'1',KEY_J,KEY_K,KEY_L,KEY_SEMICOLON,KEY_BACKSLASH,KEY_RIGHT,'2','3','a'},
  {'1',KEY_RIGHT_SHIFT,KEY_Z,KEY_X,KEY_C,KEY_V,KEY_ENTER,KEY_M,KEY_COMMA,KEY_PERIOD,KEY_SLASH,'3','1','2','3','a'},
  {'1','2',KEY_CAPS_LOCK,'1','2',KEY_B,KEY_SPACE,KEY_N,KEY_TILDE,KEY_INSERT,KEY_DELETE,KEY_BACKSPACE,'1','2',KEY_A,'z'}
  
};
byte rowPins[ROWS] = {1,2,3,4,5,6,7,8}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24}; //connect to the column pinouts of the keypad

Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

unsigned long loopCount;
unsigned long startTime;
String msg;
int x = 0;

void setup() {
    Serial.begin(9600);
    loopCount = 0;
    startTime = millis();
    msg = "";
}


void loop() {
    loopCount++;
    //char key = kpd.getKey();
    if ( (millis()-startTime)>5000 ) {
        Serial.print("Average loops per second = ");
        Serial.println(loopCount/5);
        startTime = millis();
        loopCount = 0;
    }
    
    
    // Fills kpd.key[ ] array with up-to 10 active keys.
    // Returns true if there are ANY active keys.
    if (kpd.getKeys())
    {
        for (int i=0; i<LIST_MAX; i++)   // Scan the whole key list.
        {
            if ( kpd.key[i].stateChanged )   // Only find keys that have changed state.
            {
                switch (kpd.key[i].kstate) {  // Report active key state : IDLE, PRESSED, HOLD, or RELEASED
                    case PRESSED:
                    msg = " PRESSED.";

                    
                    
                    Serial.print("Key ");
                Serial.print(kpd.key[i].kchar);
                Serial.println(msg);
                // release all the keys at the same instant
                 Keyboard.set_key1(kpd.key[i].kchar);
                Keyboard.send_now();
               
                //Keyboard.set_key1(0);
                //Keyboard.send_now();
                if (kpd.key[i].kchar == char(KEY_LEFT_SHIFT)) {
                  Serial.print("Shift ");
                  Keyboard.set_modifier(MODIFIERKEY_SHIFT);
                  Keyboard.send_now();
                  x=1;
             
        }
        
                break;
                 
                    case HOLD:
                    msg = " HOLD.";
                    //Keyboard.set_modifier(MODIFIERKEY_SHIFT);
                    //Keyboard.send_now();
                    
                    Keyboard.set_key1(kpd.key[i].kchar);
                Keyboard.send_now();
                    
                break;
                    case RELEASED:
                    msg = " RELEASED.";
                    Serial.println(msg);
                    Keyboard.set_key1(0);
                Keyboard.send_now();
                if (kpd.key[i].kchar == char(KEY_LEFT_SHIFT)) {
                  Serial.print("Shift Released ");
                  Keyboard.set_modifier(0);
                  Keyboard.send_now();
                  x=0;
                }
               
                break;
                    case IDLE:
                    msg = " IDLE.";
                    
                }
                
            }
        }
    }
}  // End ffffSsSssSssSdddDdddDddDdDdddDdDdddDddDddC
